/////////////////////////////////////////////////////////////////////////////
// Name:        tbarbase.h
// Purpose:     Base class for toolbar classes
// Author:      Julian Smart
// Modified by:
// Created:     01/02/97
// RCS-ID:      $Id: tbarbase.h,v 1.13 1999/05/31 10:47:41 RR Exp $
// Copyright:   (c) Julian Smart and Markus Holzem
// Licence:   	wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifndef _WX_TBARBASE_H_
#define _WX_TBARBASE_H_

#ifdef __GNUG__
#pragma interface "tbarbase.h"
#endif

#include "wx/setup.h"
#include "wx/defs.h"

#include "wx/bitmap.h"
#include "wx/list.h"
#include "wx/control.h"

WXDLLEXPORT_DATA(extern const wxChar*) wxToolBarNameStr;
WXDLLEXPORT_DATA(extern const wxSize) wxDefaultSize;
WXDLLEXPORT_DATA(extern const wxPoint) wxDefaultPosition;

#define wxTOOL_STYLE_BUTTON          1
#define wxTOOL_STYLE_SEPARATOR       2

#ifdef __WXGTK__
class WXDLLEXPORT wxToolBar;
#endif

class WXDLLEXPORT wxToolBarTool: public wxObject
{
  DECLARE_DYNAMIC_CLASS(wxToolBarTool)
 public:
  wxToolBarTool() {}
#ifdef __WXGTK__
  wxToolBarTool(wxToolBar *owner, 
                int theIndex = 0, const wxBitmap& bitmap1 = wxNullBitmap, const wxBitmap& bitmap2 = wxNullBitmap,
                bool toggle = FALSE, wxObject *clientData = (wxObject *) NULL, 
                const wxString& shortHelpString = "", const wxString& longHelpString = "",
                GtkWidget *pixmap = (GtkWidget *) NULL );
#else
  wxToolBarTool(int theIndex, const wxBitmap& bitmap1 = wxNullBitmap, const wxBitmap& bitmap2 = wxNullBitmap,
                bool toggle = FALSE, long xPos = -1, long yPos = -1,
                const wxString& shortHelpString = wxEmptyString, const wxString& longHelpString = wxEmptyString);
#endif
  ~wxToolBarTool ();
  inline void SetSize( long w, long h ) { m_width = w; m_height = h; }
  inline long GetWidth () const { return m_width; }
  inline long GetHeight () const { return m_height; }

public:
  int                   m_toolStyle;
  wxObject *            m_clientData;
  int                   m_index;
  long                  m_x;
  long                  m_y;
  long                  m_width;
  long                  m_height;
  bool                  m_toggleState;
  bool                  m_isToggle;
  bool                  m_deleteSecondBitmap;
  bool                  m_enabled;
  wxBitmap              m_bitmap1;
  wxBitmap              m_bitmap2;
  bool                  m_isMenuCommand;
  wxString              m_shortHelpString;
  wxString              m_longHelpString;
#ifdef __WXGTK__
  wxToolBar            *m_owner;
  GtkWidget            *m_item;
  GtkWidget            *m_pixmap;
#endif
};

class WXDLLEXPORT wxToolBarBase : public wxControl
{
  DECLARE_ABSTRACT_CLASS(wxToolBarBase)
 public:

  wxToolBarBase(void);
  ~wxToolBarBase(void);

  // Handle wxToolBar events

  // Only allow toggle if returns TRUE. Call when left button up.
  virtual bool OnLeftClick(int toolIndex, bool toggleDown);

  // Call when right button down.
  virtual void OnRightClick(int toolIndex, long x, long y);

  // Called when the mouse cursor enters a tool bitmap.
  // Argument is -1 if mouse is exiting the toolbar.
  virtual void OnMouseEnter(int toolIndex);

  // If pushedBitmap is NULL, a reversed version of bitmap is
  // created and used as the pushed/toggled image.
  // If toggle is TRUE, the button toggles between the two states.
  virtual wxToolBarTool *AddTool(int toolIndex, const wxBitmap& bitmap, const wxBitmap& pushedBitmap = wxNullBitmap,
               bool toggle = FALSE, long xPos = -1, long yPos = -1, wxObject *clientData = NULL,
               const wxString& helpString1 = wxEmptyString, const wxString& helpString2 = wxEmptyString);
  virtual void AddSeparator(void);
  virtual void ClearTools(void);

  virtual void EnableTool(int toolIndex, bool enable);
  virtual void ToggleTool(int toolIndex, bool toggle); // toggle is TRUE if toggled on
  virtual void SetToggle(int toolIndex, bool toggle); // Set this to be togglable (or not)
  virtual wxObject *GetToolClientData(int index) const;
  inline wxList& GetTools(void) const { return (wxList&) m_tools; }

  // After the toolbar has initialized, this is the size the tools take up
#if WXWXIN_COMPATIBILITY
  inline void GetMaxSize ( long * width, long * height ) const
   { wxSize maxSize(GetMaxSize()); *width = maxSize.x; *height = maxSize.y; }
#endif
  virtual wxSize GetMaxSize ( void ) const;

  virtual bool GetToolState(int toolIndex) const;
  virtual bool GetToolEnabled(int toolIndex) const;
  virtual wxToolBarTool *FindToolForPosition(long x, long y) const;

  virtual void SetToolShortHelp(int toolIndex, const wxString& helpString);
  virtual wxString GetToolShortHelp(int toolIndex) const;
  virtual void SetToolLongHelp(int toolIndex, const wxString& helpString);
  virtual wxString GetToolLongHelp(int toolIndex) const;

  virtual void SetMargins(int x, int y);
  inline void SetMargins(const wxSize& size) { SetMargins((int) size.x, (int) size.y); }
  virtual void SetToolPacking(int packing);
  virtual void SetToolSeparation(int separation);

  inline virtual wxSize GetToolMargins(void) { return wxSize(m_xMargin, m_yMargin); }
  inline virtual int GetToolPacking(void) { return m_toolPacking; }
  inline virtual int GetToolSeparation(void) { return m_toolSeparation; }

  virtual void SetToolBitmapSize(const wxSize& size) { m_defaultWidth = size.x; m_defaultHeight = size.y; };
  virtual wxSize GetToolBitmapSize(void) const { return wxSize(m_defaultWidth, m_defaultHeight); }

  // The button size (in some implementations) is bigger than the bitmap size: this returns
  // the total button size.
  virtual wxSize GetToolSize(void) const { return wxSize(m_defaultWidth, m_defaultHeight); } ;

  // Compatibility
#if WXWIN_COMPATIBILITY
  inline void SetDefaultSize(int w, int h) { SetDefaultSize(wxSize(w, h)); }
  inline long GetDefaultWidth(void) const { return m_defaultWidth; }
  inline long GetDefaultHeight(void) const { return m_defaultHeight; }
  inline int GetDefaultButtonWidth(void) const { return (int) GetDefaultButtonSize().x; };
  inline int GetDefaultButtonHeight(void) const { return (int) GetDefaultButtonSize().y; };
  virtual void SetDefaultSize(const wxSize& size) { SetToolBitmapSize(size); }
  virtual wxSize GetDefaultSize(void) const { return GetToolBitmapSize(); }
  virtual wxSize GetDefaultButtonSize(void) const { return GetToolSize(); }
#endif

  // Lay the tools out
  virtual void LayoutTools();
  
  // Add all the buttons: required for Win95.
  virtual bool CreateTools(void) { return TRUE; }

  // Calls the appropriate function after tools have been created.
  // E.g. Layout, or CreateTools.
  virtual bool Realize() = 0;

  void Command(wxCommandEvent& event);

  // SCROLLING: this has to be copied from wxScrolledWindow since wxToolBarBase
  // inherits from wxControl. This could have been put into wxToolBarSimple,
  // but we might want any derived toolbar class to be scrollable.

  // Number of pixels per user unit (0 or -1 for no scrollbar)
  // Length of virtual canvas in user units
  virtual void SetScrollbars(int horizontal, int vertical,
                             int x_length, int y_length,
                             int x_pos = 0, int y_pos = 0);

  // Physically scroll the window
  virtual void Scroll(int x_pos, int y_pos);
  virtual void GetScrollPixelsPerUnit(int *x_unit, int *y_unit) const;
  virtual void EnableScrolling(bool x_scrolling, bool y_scrolling);
  virtual void AdjustScrollbars(void);

  // Prepare the DC by translating it according to the current scroll position
  virtual void PrepareDC(wxDC& dc);

  int GetScrollPageSize(int orient) const ;
  void SetScrollPageSize(int orient, int pageSize);

  // Get the view start
  virtual void ViewStart(int *x, int *y) const;

  // Actual size in pixels when scrolling is taken into account
  virtual void GetVirtualSize(int *x, int *y) const;

  // Do the toolbar button updates (check for EVT_UPDATE_UI handlers)
  virtual void DoToolbarUpdates(void);

  inline void SetMaxRowsCols(int rows, int cols) { m_maxRows = rows; m_maxCols = cols; }
  inline int GetMaxRows() const { return m_maxRows; }
  inline int GetMaxCols() const { return m_maxCols; }

  void OnScroll(wxScrollEvent& event);
  void OnSize(wxSizeEvent& event);
  void OnIdle(wxIdleEvent& event);

 protected:
  wxList                m_tools;
//  int                   m_tilingDirection;
//  int                   m_rowsOrColumns;
  int                   m_maxRows;
  int                   m_maxCols;
  long                  m_maxWidth, m_maxHeight;
  int                   m_currentTool; // Tool where mouse currently is
  int                   m_pressedTool; // Tool where mouse pressed
  int                   m_xMargin;
  int                   m_yMargin;
  int                   m_toolPacking;
  int                   m_toolSeparation;
  long                  m_defaultWidth;
  long                  m_defaultHeight;

public:
  ////////////////////////////////////////////////////////////////////////
  //// IMPLEMENTATION
  
  // Calculate scroll increment
  virtual int CalcScrollInc(wxScrollEvent& event);

  ////////////////////////////////////////////////////////////////////////
  //// PROTECTED DATA
protected:
  int                   m_xScrollPixelsPerLine;
  int                   m_yScrollPixelsPerLine;
  bool                  m_xScrollingEnabled;
  bool                  m_yScrollingEnabled;
  int                   m_xScrollPosition;
  int                   m_yScrollPosition;
  bool                  m_calcScrolledOffset; // If TRUE, wxCanvasDC uses scrolled offsets
  int                   m_xScrollLines;
  int                   m_yScrollLines;
  int                   m_xScrollLinesPerPage;
  int                   m_yScrollLinesPerPage;

public:
    DECLARE_EVENT_TABLE()
};

#endif
    // _WX_TBARBASE_H_

